C--------------------------------------------------------------------
C--------------------------------------------------------------------
C     Main program for the standalone charybdis implementation:
C
C     -A Les houches event file is produced with the events
C     -An external parton distribution library must be used. The two 
C      possibilities are PDFLIB and LHAPDF. These options are 
C      chosen by editing the Makefile
C--------------------------------------------------------------------
C--------------------------------------------------------------------
      PROGRAM CHPROGRAM
C-----Declaration of variables---------------------------------------
      IMPLICIT NONE
      INCLUDE 'charybdis2-1.0.1.inc'
C--Les Houches run common block
      INTEGER MAXPUP
      PARAMETER(MAXPUP=100)
      INTEGER IDBMUP,PDFGUP,PDFSUP,IDWTUP,NPRUP,LPRUP
      DOUBLE PRECISION EBMUP,XSECUP,XERRUP,XMAXUP
      COMMON /HEPRUP/ IDBMUP(2),EBMUP(2),PDFGUP(2),PDFSUP(2),
     &                IDWTUP,NPRUP,XSECUP(MAXPUP),XERRUP(MAXPUP),
     &                XMAXUP(MAXPUP),LPRUP(MAXPUP)
C--event common block
      INTEGER MAXNUP
      PARAMETER (MAXNUP=500)
      INTEGER NUP,IDPRUP,IDUP,ISTUP,MOTHUP,ICOLUP
      DOUBLE PRECISION XWGTUP,SCALUP,AQEDUP,AQCDUP,PUP,VTIMUP,SPINUP
      COMMON/HEPEUP/NUP,IDPRUP,XWGTUP,SCALUP,AQEDUP,AQCDUP,
     &              IDUP(MAXNUP),ISTUP(MAXNUP),MOTHUP(2,MAXNUP),
     &              ICOLUP(2,MAXNUP),PUP(5,MAXNUP),VTIMUP(MAXNUP),
     &              SPINUP(MAXNUP)
C-----Local Variables
      INTEGER NEVENTS,NSUCCESS,MAXEV,MAXPR
      EXTERNAL CHDATA
C-----Standalone mode common block for random number generator seeds
      INTEGER NRN
      COMMON/CHRANDOM/NRN(2)
    
      MAXEV=1000
      MAXPR=10
      NSUCCESS=0 
C--------------------------------------------------------------------
C                           INITIALISATION
C
C     Please set parameters in the subroutine CHDEFAULTS below, where 
C     the default values can be changed. Use also 
C--------------------------------------------------------------------

C---CHANGE DEFAULTS---------------------(EDIT THIS ROUTINE if needed)
      CALL CHDEFAULTS(MAXEV,MAXPR)
C--------------------------------------------------------------------

C---Events Initialisation routine
      CALL UPINIT
C---User defined initial routine--------(EDIT THIS ROUTINE if needed)
      CALL CHARYBEG
C--------------------------------------------------------------------

C--------------------------------------------------------------------
C                       GENERATE EVENTS
C-------------------------------------------------------------------- 
C---Generate events
      DO 100 NEVENTS=1,MAXEV
C--------Generate event
         CALL UPEVNT
         IF(XWGTUP.EQ.0) THEN
            WRITE(*,*)'EVENT FAILED. SKIP TO NEXT EVENT'
            GOTO 100
         END IF
         NSUCCESS=NSUCCESS+1
C--------User analysis routine----------(EDIT THIS ROUTINE if needed)
         CALL CHANAL
C--------------------------------------------------------------------

C--------Print out BH history
         CALL CHPRINT(NSUCCESS,MAXPR,MAXEV)
  100 CONTINUE

C---User's terminal calculations--------(EDIT THIS ROUTINE if needed)
      CALL CHAEND

C--------------------------------------------------------------------
      WRITE(*,*)'Total number of events generated =',NSUCCESS
      STOP
      END

C----------------------------------------------------------------------
      SUBROUTINE CHDEFAULTS(MAXEV,MAXPR)
C     Routine for changing charybdis defaults. Make your changes here
C----------------------------------------------------------------------
      IMPLICIT NONE
      INCLUDE 'charybdis2-1.0.1.inc'
C--Les Houches run common block
      INTEGER MAXPUP
      PARAMETER(MAXPUP=100)
      INTEGER IDBMUP,PDFGUP,PDFSUP,IDWTUP,NPRUP,LPRUP
      DOUBLE PRECISION EBMUP,XSECUP,XERRUP,XMAXUP
      COMMON /HEPRUP/ IDBMUP(2),EBMUP(2),PDFGUP(2),PDFSUP(2),
     &                IDWTUP,NPRUP,XSECUP(MAXPUP),XERRUP(MAXPUP),
     &                XMAXUP(MAXPUP),LPRUP(MAXPUP)
C--event common block
      INTEGER MAXNUP
      PARAMETER (MAXNUP=500)
      INTEGER NUP,IDPRUP,IDUP,ISTUP,MOTHUP,ICOLUP
      DOUBLE PRECISION XWGTUP,SCALUP,AQEDUP,AQCDUP,PUP,VTIMUP,SPINUP
      COMMON/HEPEUP/NUP,IDPRUP,XWGTUP,SCALUP,AQEDUP,AQCDUP,
     &              IDUP(MAXNUP),ISTUP(MAXNUP),MOTHUP(2,MAXNUP),
     &              ICOLUP(2,MAXNUP),PUP(5,MAXNUP),VTIMUP(MAXNUP),
     &              SPINUP(MAXNUP)
C--Max number of events and printed events
      INTEGER MAXEV,MAXPR
C--Random numbers
      INTEGER NRN
      COMMON/CHRANDOM/NRN(2)

C-----Reset Charybdis Parameters here (optional)
      TOTDIM=8
      MINMSS=5000D0
      MAXMSS=14000D0
      MSSDEF=3
      MPLNCK=1000D0
      MAXEV=100

C-----Reset Les houches common blocks parameters here (optional)
      IDBMUP(1)=2212
      IDBMUP(2)=2212
      EBMUP(1)=7000.0D0
      EBMUP(2)=7000.0D0
C-----Reset XML Les Houches Event file name here (optional. Note: this is 8 characters long)
      LHEFILENAME='lhouches'
C---Seeds for the random number generator
      NRN(1)=92383
      NRN(2)=12837
C-----Number of events to print on screen (optional)
      MAXPR=10

      END

C----------------------------------------------------------------------
      SUBROUTINE CHARYBEG
C     USER'S ROUTINE FOR INITIALIZATION
C----------------------------------------------------------------------
      IMPLICIT NONE
      INCLUDE 'charybdis2-1.0.1.inc'
C--Les Houches run common block
      INTEGER MAXPUP
      PARAMETER(MAXPUP=100)
      INTEGER IDBMUP,PDFGUP,PDFSUP,IDWTUP,NPRUP,LPRUP
      DOUBLE PRECISION EBMUP,XSECUP,XERRUP,XMAXUP
      COMMON /HEPRUP/ IDBMUP(2),EBMUP(2),PDFGUP(2),PDFSUP(2),
     &                IDWTUP,NPRUP,XSECUP(MAXPUP),XERRUP(MAXPUP),
     &                XMAXUP(MAXPUP),LPRUP(MAXPUP)

C---Open LES HOUCHES EVENT FILE to write event
      CALL SYSTEM('rm '//LHEFILENAME//'.xml;')
      OPEN(UNIT=10,FILE=LHEFILENAME//'.xml',STATUS='NEW')
C------Write header information        
      WRITE(10,1)'<LesHouchesEvents version="1.0">'
      WRITE(10,1)'<!--'
      WRITE(10,1)'File generated with Charybdis2.001'
      WRITE(10,1)'-->'
      WRITE(10,1)'<init>'
      WRITE(10,2)IDBMUP,EBMUP,PDFGUP,PDFSUP,IDWTUP,1
      WRITE(10,3)XSECUP(1),XERRUP(1),1D0,LPRUP(1)
      WRITE(10,1)'</init>'
 1    FORMAT(A)
 2    FORMAT(2(TR1,I5),2(TR1,E11.2),6(TR1,I5))
 3    FORMAT(3(TR1,E11.2),1(TR1,I5))

      CALL SYSTEM('rm histories.xml;')
      OPEN(UNIT=11,FILE='histories.xml',STATUS='NEW')
C------Write header information     
      WRITE(11,1)'<BHhistories>'
      WRITE(11,1)'<!--'
      WRITE(11,1)'File generated with Charybdis2.001'
      WRITE(11,1)'-->'
      WRITE(11,1)'<init>'
      WRITE(11,4)IDBMUP,EBMUP,PDFGUP,PDFSUP,IDWTUP,1
      WRITE(11,5)XSECUP(1),XERRUP(1),XMAXUP(1),LPRUP(1)
      WRITE(11,1)'</init>'
 4    FORMAT(2(TR1,I5),2(TR1,E11.2),6(TR1,I5))
 5    FORMAT(3(TR1,E11.2),1(TR1,I5))

      END

C----------------------------------------------------------------------
      SUBROUTINE CHANAL
C     USER'S ROUTINE TO ANALYSE DATA FROM EVENT
C----------------------------------------------------------------------
      IMPLICIT NONE
      INCLUDE 'charybdis2-1.0.1.inc'
C--Les Houches run common block
      INTEGER MAXPUP
      PARAMETER(MAXPUP=100)
      INTEGER IDBMUP,PDFGUP,PDFSUP,IDWTUP,NPRUP,LPRUP
      DOUBLE PRECISION EBMUP,XSECUP,XERRUP,XMAXUP
      COMMON /HEPRUP/ IDBMUP(2),EBMUP(2),PDFGUP(2),PDFSUP(2),
     &                IDWTUP,NPRUP,XSECUP(MAXPUP),XERRUP(MAXPUP),
     &                XMAXUP(MAXPUP),LPRUP(MAXPUP)
C--event common block
      INTEGER MAXNUP
      PARAMETER (MAXNUP=500)
      INTEGER NUP,IDPRUP,IDUP,ISTUP,MOTHUP,ICOLUP
      DOUBLE PRECISION XWGTUP,SCALUP,AQEDUP,AQCDUP,PUP,VTIMUP,SPINUP
      COMMON/HEPEUP/NUP,IDPRUP,XWGTUP,SCALUP,AQEDUP,AQCDUP,
     &              IDUP(MAXNUP),ISTUP(MAXNUP),MOTHUP(2,MAXNUP),
     &              ICOLUP(2,MAXNUP),PUP(5,MAXNUP),VTIMUP(MAXNUP),
     &              SPINUP(MAXNUP)
C--Local variables
      INTEGER I,J
      DOUBLE PRECISION MODJ
      DOUBLE PRECISION DELTA2,P02

C--Write out event in the LES HOUCHES EVENT FILE
      IF(XWGTUP.NE.0)THEN

      WRITE(10,1)'<event>'
      IF(MJLOST)THEN
         WRITE(10,2)NUP+2,LPRUP(1),1D0,SCALUP,1./137.,0.118
      ELSE
         WRITE(10,2)NUP+1,LPRUP(1),1D0,SCALUP,1./137.,0.118            
      END IF
C-----Put incoming partons in the event record
      DO J=1,2
         WRITE(10,3)IDUP(J),ISTUP(J),MOTHUP(1,J),MOTHUP(2,J),
     &ICOLUP(1,J),ICOLUP(2,J),(PUP(I,J),I=1,5),VTIMUP(J),SPINUP(J)
      END DO

C-----Put Black hole in the event record
      MODJ=SQRT(SBHD(1,1)**2+SBHD(2,1)**2+SBHD(3,1)**2)
      IF(MODJ.GT.0.1D0)THEN
         WRITE(10,3)40,2,1,2,0,0,(PBHD(I,1),I=1,5),0,
     &(SBHD(1,1)*PBHD(1,1)+SBHD(2,1)*PBHD(2,1)+SBHD(3,1)*PBHD(3,1))/
     &SQRT(PBHD(1,1)**2+PBHD(2,1)**2+PBHD(3,1)**2)/MODJ
      ELSE
         WRITE(10,3)40,2,1,2,0,0,(PBHD(I,1),I=1,5),0,9D0
      END IF

      DO J=3,NUP
         IF(IDUP(J).NE.39)THEN
            WRITE(10,3)IDUP(J),ISTUP(J),MOTHUP(1,J),MOTHUP(2,J),
     &ICOLUP(1,J),ICOLUP(2,J),(PUP(I,J),I=1,5),VTIMUP(J),SPINUP(J)
         ELSE
C-----Split total gravitational radiation into two fictitious massless gravitons (note that this is missing energy anyway)
            WRITE(10,3)IDUP(J),ISTUP(J),MOTHUP(1,J),MOTHUP(2,J),
     &ICOLUP(1,J),ICOLUP(2,J),
     &PUP(1,J)/2+PUP(5,J)*PUP(2,J)/SQRT(PUP(1,J)**2+PUP(2,J)**2),
     &PUP(2,J)/2-PUP(5,J)*PUP(1,J)/SQRT(PUP(1,J)**2+PUP(2,J)**2),
     &PUP(3,J)/2,PUP(4,J)/2,0D0,VTIMUP(J),SPINUP(J)   
            WRITE(10,3)IDUP(J),ISTUP(J),MOTHUP(1,J),MOTHUP(2,J),
     &ICOLUP(1,J),ICOLUP(2,J),
     &PUP(1,J)/2-PUP(5,J)*PUP(2,J)/SQRT(PUP(1,J)**2+PUP(2,J)**2),
     &PUP(2,J)/2+PUP(5,J)*PUP(1,J)/SQRT(PUP(1,J)**2+PUP(2,J)**2),
     &PUP(3,J)/2,PUP(4,J)/2,0D0,VTIMUP(J),SPINUP(J) 
         END IF
      END DO
      WRITE(10,1)'</event>'

C------Write BH histories in a similar file
      WRITE(11,1)'<event>'
      WRITE(11,*)NBHD
      DO J=1,NBHD
         WRITE(11,4)(PBHD(I,J),I=1,5),(SBHD(I,J),I=1,3),RHBHD(J),
     &OBBHD(J),THBHD(J),NFLUXBHD(J)
      END DO
      WRITE(11,1)'</event>'

 1    FORMAT(A)
 2    FORMAT(2(TR1,I5),4(TR1,E17.10))
 3    FORMAT(6(TR1,I5),5(TR1,E17.10),2(TR1,F3.0))
 4    FORMAT(12(TR1,E17.10))
      END IF

      END

C----------------------------------------------------------------------
      SUBROUTINE CHPRINT(N,MAXPR,MAXEVN)
C     PRINT OUT BH HISTORY
C----------------------------------------------------------------------
      IMPLICIT NONE
      INCLUDE 'charybdis2-1.0.1.inc'
      INTEGER N,MAXPR,MAXEVN
C---LOCAL VARIABLES
      INTEGER I,J

      IF (N.LE.MAXPR) THEN
         WRITE(*,*)
         WRITE(*,*)'                          ---BLACK HOLE HISTORY---'
         WRITE(*,*)
         WRITE(*,10)'NBHD','PBHX','PBHY','PBHZ','EBH','MBH','EEM',
     &'OMEGA*','S','SBHX','SBHY','SBHZ','A*','THBH','TSTAR'
         DO J=1,NBHD-1
            WRITE(*,11)J,(PBHD(I,J),I=1,5),PBHD(5,J)-PBHD(5,J+1),
     &(PBHD(5,J)-PBHD(5,J+1))*RHBHD(J),
     &SQRT(SBHD(1,J)**2+SBHD(2,J)**2+SBHD(3,J)**2)
     &,(SBHD(I,J),I=1,3),OBBHD(J)/RHBHD(J),THBHD(J),
     &1D0/NFLUXBHD(J)/RHBHD(J)
         ENDDO
         WRITE(*,12)J,(PBHD(I,NBHD),I=1,5),'------','------',
     &SQRT(SBHD(1,NBHD)**2+SBHD(2,NBHD)**2+SBHD(3,NBHD)**2),
     &(SBHD(I,NBHD),I=1,3),OBBHD(NBHD)/RHBHD(NBHD),THBHD(NBHD),'------'

      ENDIF


 10   FORMAT(A4,6(TR1,A8),5(TR1,A6),(TR1,A8),2(TR1,A8))
 11   FORMAT(I4,6(TR1,F8.1),5(TR1,F6.2),1(TR1,F8.2),1(TR1,F8.1),
     &1(TR1,E8.2))
 12   FORMAT(I4,5(TR1,F8.1),(TR1,A8),(TR1,A6),4(TR1,F6.2),(TR1,F8.2)
     &,1(TR1,F8.1),(TR1,A8))

      IF((N/(MAXEVN/10.)-10*N/MAXEVN).EQ.0)WRITE(*,*)N,'EVENTS DONE'
      END

C----------------------------------------------------------------------
      SUBROUTINE CHAEND
C     USER'S ROUTINE FOR TERMINAL CALCULATIONS, HISTOGRAM OUTPUT, ETC
C----------------------------------------------------------------------
      IMPLICIT NONE
      INCLUDE 'charybdis2-1.0.1.inc'

C---Finish LES HOUCHES EVENT FILE and close it
      WRITE(10,1)'</LesHouchesEvents>'
      CLOSE(UNIT=10)

C---Finish Histories FILE and close it
      WRITE(11,1)'</BHhistories>'
      CLOSE(UNIT=11)
 1    FORMAT(A)

      END


