C--------------------------------------------------------------------
C--------------------------------------------------------------------
C     Main program for the herwig implementation
C
C     -A Les houches event file is produced with the events
C     -An external parton distribution library can be used. The two 
C      possibilities are PDFLIB and LHAPDF. These options are 
C      chosen by editing the Makefile
C--------------------------------------------------------------------
C--------------------------------------------------------------------  
      PROGRAM HWIGPR
C-----Declaration of variables---------------------------------------
C---COMMON BLOCKS ARE INCLUDED AS FILE HERWIG65.INC
      INCLUDE 'HERWIG65.INC'
      INCLUDE 'charybdis2-1.0.1.inc'
C--Les Houches run common block
      INTEGER MAXPUP
      PARAMETER(MAXPUP=100)
      INTEGER IDBMUP,PDFGUP,PDFSUP,IDWTUP,NPRUP,LPRUP
      DOUBLE PRECISION EBMUP,XSECUP,XERRUP,XMAXUP
      COMMON /HEPRUP/ IDBMUP(2),EBMUP(2),PDFGUP(2),PDFSUP(2),
     &                IDWTUP,NPRUP,XSECUP(MAXPUP),XERRUP(MAXPUP),
     &                XMAXUP(MAXPUP),LPRUP(MAXPUP)
C--event common block
      INTEGER MAXNUP
      PARAMETER (MAXNUP=500)
      INTEGER NUP,IDPRUP,IDUP,ISTUP,MOTHUP,ICOLUP
      DOUBLE PRECISION XWGTUP,SCALUP,AQEDUP,AQCDUP,PUP,VTIMUP,SPINUP
      COMMON/HEPEUP/NUP,IDPRUP,XWGTUP,SCALUP,AQEDUP,AQCDUP,
     &              IDUP(MAXNUP),ISTUP(MAXNUP),MOTHUP(2,MAXNUP),
     &              ICOLUP(2,MAXNUP),PUP(5,MAXNUP),VTIMUP(MAXNUP),
     &              SPINUP(MAXNUP)
C--Local
      INTEGER N,I,J,IUP,IHEP,ID,POLN
      DOUBLE PRECISION HWRUNI
      EXTERNAL HWUDAT,HWRUNI
C--------------------------------------------------------------------
C                           INITIALISATION 
C
C     Some parameters need to be initialised here others may or may 
C     not be set (defaults will be used if not)
C--------------------------------------------------------------------

C--------------Compulsory initialisation-----------------------------
C---MAX NUMBER OF EVENTS THIS RUN------------(MAKE CHANGES IF NEEDED)
      MAXEV=100
C---BEAM PARTICLES
      IDBMUP(1)=2212
      IDBMUP(2)=2212
C---BEAM MOMENTA
      EBMUP(1)=7000D0
      EBMUP(2)=7000D0
C-----------------------(END OF CHANGES)-----------------------------

C---PROCESS (Do not change this value (must be  <0)!!!!!)
      IPROC=-100
      CALL HWUIDT(1,IDBMUP(1),I,PART1)
      CALL HWUIDT(1,IDBMUP(2),I,PART2)
      PBEAM1=EBMUP(1)
      PBEAM2=EBMUP(2)
C---INITIALISE OTHER COMMON BLOCKS
      CALL HWIGIN
C---Herwig optional change of defaults--------(MAKE CHANGES IF NEEDED)
C---Random number generator seeds
      NRN(1)=9685569
      NRN(2)=1393138
C---Print vertex information
      PRVTX = .FALSE.
C---Maximum number of events to print
      MAXPR = 10
C-----------------------(END OF CHANGES)-----------------------------

C---Change CHARYBDIS defaults------------(EDIT THIS ROUTINE IF NEEDED)
      CALL CHDEFAULTS
C--------------------------------------------------------------------

C--------------------------------------------------------------------
C                    Other HERWIG initialisation  
C--------------------------------------------------------------------
      
      IF (RMSTAB) THEN
C---DEFINE STABLE BLACK HOLE REMNANTS
         NRES=NRES+1
         RNAME(NRES)='Remnant0'
         IDPDG(NRES)=50
         ICHRG(NRES)=0
         NRES=NRES+1
         RNAME(NRES)='Remnant+'
         IDPDG(NRES)=51
         ICHRG(NRES)=1
         NRES=NRES+1
         RNAME(NRES)='Remnant-'
         IDPDG(NRES)=-51
         ICHRG(NRES)=-1
      ENDIF
C---DEFINE BLACK HOLE
      NRES=NRES+1
      RNAME(NRES)='BlacHole'
      WRITE(*,*)'NRES=',NRES
      IDPDG(NRES)=40

C---COMPUTE PARAMETER-DEPENDENT CONSTANTS
      CALL HWUINC
      AVWGT =XSECUP(1)
      WGTMAX=XMAXUP(1)
      EVWGT=AVWGT 
C---CALL HWUSTA TO MAKE ANY PARTICLE STABLE
      CALL HWUSTA('PI0     ')
C---USER'S INITIAL CALCULATION------------(Edit this subroutine if needed)
      CALL HWABEG
C-------------------------------------------------------------------------
C---INITIALISE ELEMENTARY PROCESS
      CALL HWEINI
      MAXER=MAX(10,MAXEV/10)
C-------------------------------------------------------------------------
C                               GENERATE EVENTS
C-------------------------------------------------------------------------
C---LOOP OVER EVENTS
      DO 100 N=1,MAXEV
C---INITIALISE EVENT
      CALL HWUINE
C---GENERATE HARD SUBPROCESS
      CALL HWEPRO
C---GET POLARIZATIONS
      IUP=3
      DO IHEP=JDAHEP(1,6),JDAHEP(2,6)
         ID=IDHEP(IHEP)
         IF (IDUP(IUP).EQ.ID) THEN
            CALL HWVZRO(3,RHOHEP(1,IHEP))
            POLN=SPINUP(IUP)
            IF (POLN.LT.1.1D0) THEN
               IF (POLN.LT.ZERO) THEN
                  RHOHEP(1,IHEP)=ONE
               ELSEIF (POLN.GT.ZERO) THEN
                  RHOHEP(3,IHEP)=ONE
               ELSE
                  RHOHEP(2,IHEP)=ONE
               ENDIF
            ENDIF
         ELSE
            PRINT *,' WARNING: PARTICLE NOT FOUND IN HEPEUP'
         ENDIF
         IUP=IUP+1
      ENDDO
      
      IF (MJLOST) THEN
C--Put BH in event record
         NHEP=NHEP+1
         CALL CHVDIF(5,PHEP(1,6),PHEP(1,7),PHEP(1,NHEP))
         CALL CHUMAS(PHEP(1,NHEP))
         IDHW(NHEP)=NRES
         IDHEP(NHEP)=40
         ISTHEP(NHEP)=2      
         JMOHEP(1,NHEP) = 6
         JMOHEP(2,NHEP) = NHEP
      ELSEIF (IDHEP(6).EQ.0) THEN
C--RELABEL HARD AS BLACK HOLE (UNLESS REMANT IS IMEDIATELY OBTAINED AFTER FORMATION)
         IDHW(6)=NRES
         IDHEP(6)=40
      ENDIF  
C---GENERATE PARTON CASCADES
      CALL HWBGEN
C---DO HEAVY OBJECT DECAYS
      CALL HWDHOB
C---DO CLUSTER FORMATION
      CALL HWCFOR
C---DO CLUSTER DECAYS
      CALL HWCDEC
C---DO UNSTABLE PARTICLE DECAYS
      CALL HWDHAD
C---DO HEAVY FLAVOUR HADRON DECAYS
      CALL HWDHVY
C---ADD SOFT UNDERLYING EVENT IF NEEDED
      CALL HWMEVT
C---FINISH EVENT
      CALL HWUFNE
C---USER'S EVENT ANALYSS-------------------(EDIT THIS ROUTINE if needed)
      CALL HWANAL
C-----------------------------------------------------------------------

C--------Print out BH history
      CALL CHPRINT(N,MAXPR,MAXEV)
  100 CONTINUE

C-----------------------------------------------------------------------
C                           TERMINATE RUN
C-----------------------------------------------------------------------

C---TERMINATE ELEMENTARY PROCESS
      CALL HWEFIN
C---USER'S TERMINAL CALCULATIONS----------(EDIT THIS ROUTINE if needed)
      CALL HWAEND
C----------------------------------------------------------------------

      STOP
      END

C----------------------------------------------------------------------
C----------------------------------------------------------------------
C                       USER DEFINED ROUTINES
C----------------------------------------------------------------------
C----------------------------------------------------------------------

C----------------------------------------------------------------------
      SUBROUTINE CHDEFAULTS
C     Routine for changing charybdis defaults. Make your changes here
C----------------------------------------------------------------------
      IMPLICIT NONE
      INCLUDE 'charybdis2-1.0.1.inc'
C--Les Houches run common block
      INTEGER MAXPUP
      PARAMETER(MAXPUP=100)
      INTEGER IDBMUP,PDFGUP,PDFSUP,IDWTUP,NPRUP,LPRUP
      DOUBLE PRECISION EBMUP,XSECUP,XERRUP,XMAXUP
      COMMON /HEPRUP/ IDBMUP(2),EBMUP(2),PDFGUP(2),PDFSUP(2),
     &                IDWTUP,NPRUP,XSECUP(MAXPUP),XERRUP(MAXPUP),
     &                XMAXUP(MAXPUP),LPRUP(MAXPUP)
C--event common block
      INTEGER MAXNUP
      PARAMETER (MAXNUP=500)
      INTEGER NUP,IDPRUP,IDUP,ISTUP,MOTHUP,ICOLUP
      DOUBLE PRECISION XWGTUP,SCALUP,AQEDUP,AQCDUP,PUP,VTIMUP,SPINUP
      COMMON/HEPEUP/NUP,IDPRUP,XWGTUP,SCALUP,AQEDUP,AQCDUP,
     &              IDUP(MAXNUP),ISTUP(MAXNUP),MOTHUP(2,MAXNUP),
     &              ICOLUP(2,MAXNUP),PUP(5,MAXNUP),VTIMUP(MAXNUP),
     &              SPINUP(MAXNUP)

C-----Reset Charybdis Parameters here (optional)
      TOTDIM=8
      MINMSS=10000D0
      MAXMSS=14000D0
      MSSDEF=3
      MPLNCK=1000D0
C-----Reset XML Les Houches Event file name here (optional. Note: this is 8 characters long)
      LHEFILENAME='lhouches'

      END

C----------------------------------------------------------------------
      SUBROUTINE HWABEG
C     USER'S ROUTINE FOR INITIALIZATION
C----------------------------------------------------------------------
      INCLUDE 'HERWIG65.INC'
      INCLUDE 'charybdis2-1.0.1.inc'
C--Les Houches run common block
      INTEGER MAXPUP
      PARAMETER(MAXPUP=100)
      INTEGER IDBMUP,PDFGUP,PDFSUP,IDWTUP,NPRUP,LPRUP
      DOUBLE PRECISION EBMUP,XSECUP,XERRUP,XMAXUP
      COMMON /HEPRUP/ IDBMUP(2),EBMUP(2),PDFGUP(2),PDFSUP(2),
     &                IDWTUP,NPRUP,XSECUP(MAXPUP),XERRUP(MAXPUP),
     &                XMAXUP(MAXPUP),LPRUP(MAXPUP)

C---Open LES HOUCHES EVENT FILE to write event
      CALL SYSTEM('rm '//LHEFILENAME//'.xml;')
      OPEN(UNIT=10,FILE=LHEFILENAME//'.xml',STATUS='NEW')
C------Write header information        
      WRITE(10,1)'<LesHouchesEvents version="1.0">'
      WRITE(10,1)'<!--'
      WRITE(10,1)'File generated with Charybdis2.001'
      WRITE(10,1)'-->'
      WRITE(10,1)'<init>'
      WRITE(10,2)IDBMUP,EBMUP,PDFGUP,PDFSUP,IDWTUP,1
      WRITE(10,3)XSECUP(1),XERRUP(1),1D0,LPRUP(1)
      WRITE(10,1)'</init>'
 1    FORMAT(A)
 2    FORMAT(2(TR1,I5),2(TR1,E11.2),6(TR1,I5))
 3    FORMAT(3(TR1,E11.2),1(TR1,I5))


      END

C----------------------------------------------------------------------
      SUBROUTINE HWANAL
C     USER'S ROUTINE TO ANALYSE DATA FROM EVENT
C----------------------------------------------------------------------
      INCLUDE 'HERWIG65.INC'
      INCLUDE 'charybdis2-1.0.1.inc'
C--Les Houches run common block
      INTEGER MAXPUP
      PARAMETER(MAXPUP=100)
      INTEGER IDBMUP,PDFGUP,PDFSUP,IDWTUP,NPRUP,LPRUP
      DOUBLE PRECISION EBMUP,XSECUP,XERRUP,XMAXUP
      COMMON /HEPRUP/ IDBMUP(2),EBMUP(2),PDFGUP(2),PDFSUP(2),
     &                IDWTUP,NPRUP,XSECUP(MAXPUP),XERRUP(MAXPUP),
     &                XMAXUP(MAXPUP),LPRUP(MAXPUP)
C--event common block
      INTEGER MAXNUP
      PARAMETER (MAXNUP=500)
      INTEGER NUP,IDPRUP,IDUP,ISTUP,MOTHUP,ICOLUP
      DOUBLE PRECISION XWGTUP,SCALUP,AQEDUP,AQCDUP,PUP,VTIMUP,SPINUP
      COMMON/HEPEUP/NUP,IDPRUP,XWGTUP,SCALUP,AQEDUP,AQCDUP,
     &              IDUP(MAXNUP),ISTUP(MAXNUP),MOTHUP(2,MAXNUP),
     &              ICOLUP(2,MAXNUP),PUP(5,MAXNUP),VTIMUP(MAXNUP),
     &              SPINUP(MAXNUP)
C--Local variables
      INTEGER I,J

C--Write out event in the LES HOUCHES EVENT FILE
      IF(XWGTUP.NE.0)THEN

      WRITE(10,1)'<event>'
      IF(MJLOST)THEN
         WRITE(10,2)NUP+1,LPRUP(1),1D0,SCALUP,1./137.,0.118
      ELSE
         WRITE(10,2)NUP,LPRUP(1),1D0,SCALUP,1./137.,0.118            
      END IF
      DO J=1,NUP
         IF(IDUP(J).NE.39)THEN
            WRITE(10,3)IDUP(J),ISTUP(J),MOTHUP(1,J),MOTHUP(2,J),
     &ICOLUP(1,J),ICOLUP(2,J),(PUP(I,J),I=1,5),VTIMUP(J),SPINUP(J)
         ELSE
C-----Split total gravitational radiation into two fictitious massless gravitons (note that this is missing energy anyway)
            WRITE(10,3)IDUP(J),ISTUP(J),MOTHUP(1,J),MOTHUP(2,J),
     &ICOLUP(1,J),ICOLUP(2,J),
     &PUP(1,J)/2+PUP(5,J)*PUP(2,J)/SQRT(PUP(1,J)**2+PUP(2,J)**2),
     &PUP(2,J)/2-PUP(5,J)*PUP(1,J)/SQRT(PUP(1,J)**2+PUP(2,J)**2),
     &PUP(3,J)/2,PUP(4,J)/2,0D0,VTIMUP(J),SPINUP(J)   
            WRITE(10,3)IDUP(J),ISTUP(J),MOTHUP(1,J),MOTHUP(2,J),
     &ICOLUP(1,J),ICOLUP(2,J),
     &PUP(1,J)/2-PUP(5,J)*PUP(2,J)/SQRT(PUP(1,J)**2+PUP(2,J)**2),
     &PUP(2,J)/2+PUP(5,J)*PUP(1,J)/SQRT(PUP(1,J)**2+PUP(2,J)**2),
     &PUP(3,J)/2,PUP(4,J)/2,0D0,VTIMUP(J),SPINUP(J) 
         END IF
      END DO
         WRITE(10,1)'</event>'
 1       FORMAT(A)
 2       FORMAT(2(TR1,I5),4(TR1,E17.10))
 3       FORMAT(6(TR1,I5),5(TR1,E17.10),2(TR1,F3.0))

      END IF

      END

C----------------------------------------------------------------------
      SUBROUTINE CHPRINT(N,MAXPR,MAXEVN)
C     PRINT OUT BH HISTORY
C----------------------------------------------------------------------
      IMPLICIT NONE
      INCLUDE 'charybdis2-1.0.1.inc'
      INTEGER N,MAXPR,MAXEVN
C---LOCAL VARIABLES
      INTEGER I,J

      IF (N.LE.MAXPR) THEN
         WRITE(*,*)
         WRITE(*,*)'                          ---BLACK HOLE HISTORY---'
         WRITE(*,*)
         WRITE(*,1)'NBHD','PBHX','PBHY','PBHZ','EBH','MBH','EEM',
     &'OMEGA*','S','SBHX','SBHY','SBHZ','A*','THBH','TSTAR'
         DO J=1,NBHD-1
            WRITE(*,2)J,(PBHD(I,J),I=1,5),PBHD(5,J)-PBHD(5,J+1),
     &(PBHD(5,J)-PBHD(5,J+1))*RHBHD(J),
     &SQRT(SBHD(1,J)**2+SBHD(2,J)**2+SBHD(3,J)**2)
     &,(SBHD(I,J),I=1,3),OBBHD(J)/RHBHD(J),THBHD(J),
     &1D0/NFLUXBHD(J)/RHBHD(J)
         ENDDO
         WRITE(*,3)J,(PBHD(I,NBHD),I=1,5),'------','------',
     &SQRT(SBHD(1,NBHD)**2+SBHD(2,NBHD)**2+SBHD(3,NBHD)**2),
     &(SBHD(I,NBHD),I=1,3),OBBHD(NBHD)/RHBHD(NBHD),THBHD(NBHD),'------'

      ENDIF


 1    FORMAT(A4,6(TR1,A8),5(TR1,A6),(TR1,A8),2(TR1,A8))
 2    FORMAT(I4,6(TR1,F8.1),5(TR1,F6.2),1(TR1,F8.2),1(TR1,F8.1),
     &1(TR1,E8.2))
 3    FORMAT(I4,5(TR1,F8.1),(TR1,A8),(TR1,A6),4(TR1,F6.2),(TR1,F8.2)
     &,1(TR1,F8.1),(TR1,A8))

      IF((N/(MAXEVN/10.)-10*N/MAXEVN).EQ.0)WRITE(*,*)N,'EVENTS DONE'
      END

C----------------------------------------------------------------------
      SUBROUTINE HWAEND
C     USER'S ROUTINE FOR TERMINAL CALCULATIONS, HISTOGRAM OUTPUT, ETC
C----------------------------------------------------------------------
      INCLUDE 'HERWIG65.INC'
      INCLUDE 'charybdis2-1.0.1.inc'

C---Finish LES HOUCHES EVENT FILE and close it
      WRITE(10,1)'</LesHouchesEvents>'
      CLOSE(UNIT=10)
 1    FORMAT(A)

      END
